# -*- coding: utf-8 -*-

# Copyright 2013 Vincent Jacques
# vincent@vincent-jacques.net

# This file is part of MockMockMock. http://jacquev6.github.com/MockMockMock

# MockMockMock is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

# MockMockMock is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public License along with MockMockMock.  If not, see <http://www.gnu.org/licenses/>.

from OrderingPolicies import OrderedOrderingPolicy, UnorderedOrderingPolicy
from CompletionPolicies import AllCompletionPolicy, AnyCompletionPolicy, RepeatedCompletionPolicy, ExactlyOneCompletionPolicy


class ExpectationWrapper(object):
    def __init__(self, expectation):
        self.__expectation = expectation
        self.__parent = None
        self.__called = False

    @property
    def name(self):
        return self.__expectation.name

    def expectsCall(self):
        return self.__expectation.expectsCall()

    def checkCall(self, args, kwds):
        return self.__expectation.checkCall(args, kwds)

    def checkName(self, name):
        return self.__expectation.checkName(name)

    def setParent(self, parent):
        assert(self.__parent is None)
        self.__parent = parent

    def getCurrentPossibleExpectations(self):
        if self.__called:
            return []
        else:
            return [self]

    def requiresMoreCalls(self):
        if self.__called:
            return False
        else:
            return True

    def getRequiredCallsExamples(self):
        return [self.__expectation.name]

    @property
    def called(self):
        return self.__called

    def call(self):
        self.__called = True
        self.__parent.markExpectationCalled(self)
        return self.__expectation.call(), self.__parent.rewindGroups()

    def resetCall(self):
        self.__called = False


class ExpectationGroup:
    def __init__(self, ordering, completion, sticky):
        self.__ordering = ordering
        self.__completion = completion
        self.__sticky = sticky
        self.__parent = None
        self.__expectations = []

    def setParent(self, parent):
        assert(self.__parent is None)
        self.__parent = parent

    @property
    def parent(self):
        assert(self.__parent is not None)
        return self.__parent

    def addExpectation(self, expectation):
        wrapper = ExpectationWrapper(expectation)
        wrapper.setParent(self)
        self.__expectations.append(wrapper)

    def addGroup(self, group):
        group.setParent(self)
        self.__expectations.append(group)

    def getCurrentPossibleExpectations(self):
        if self.__completion.acceptsMoreCalls(self.__expectations):
            return self.__ordering.getCurrentPossibleExpectations(self.__expectations)
        else:
            return []

    def requiresMoreCalls(self):
        return self.__completion.requiresMoreCalls(self.__expectations)

    def getRequiredCallsExamples(self):
        return self.__completion.getRequiredCallsExamples(self.__expectations)

    def rewindGroups(self):
        if self.__shallStick():
            return self
        else:
            return self.__parent.rewindGroups()

    def __shallStick(self):
        if self.__parent is None:
            return True
        if self.__sticky:
            return len(self.getCurrentPossibleExpectations()) != 0
        return False

    def markExpectationCalled(self, expectation):
        self.__completion.markExpectationCalled(self.__expectations, expectation)

    def resetCall(self):
        for e in self.__expectations:
            e.resetCall()


def makeGroup(ordering, completion, sticky):
    class Group(ExpectationGroup):
        def __init__(self):
            ExpectationGroup.__init__(self, ordering(), completion(), sticky)

    return Group


OrderedExpectationGroup = makeGroup(OrderedOrderingPolicy, AllCompletionPolicy, False)
UnorderedExpectationGroup = makeGroup(UnorderedOrderingPolicy, AllCompletionPolicy, False)
AtomicExpectationGroup = makeGroup(OrderedOrderingPolicy, AllCompletionPolicy, True)
OptionalExpectationGroup = makeGroup(OrderedOrderingPolicy, AnyCompletionPolicy, False)
AlternativeExpectationGroup = makeGroup(UnorderedOrderingPolicy, ExactlyOneCompletionPolicy, False)
RepeatedExpectationGroup = makeGroup(OrderedOrderingPolicy, RepeatedCompletionPolicy, False)



# -*- coding: utf-8 -*-

# Copyright 2013 Vincent Jacques
# vincent@vincent-jacques.net

# This file is part of MockMockMock. http://jacquev6.github.com/MockMockMock

# MockMockMock is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

# MockMockMock is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public License along with MockMockMock.  If not, see <http://www.gnu.org/licenses/>.

import unittest
import collections

import MockMockMock


def isCallable(x):
    return isinstance(x, collections.Callable)


class TestException(Exception):
    pass


class PublicInterface(unittest.TestCase):
    def setUp(self):
        unittest.TestCase.setUp(self)
        self.mocks = MockMockMock.Engine()
        self.myMock = self.mocks.create("myMock")

    def testMockMockMock(self):
        self.assertEqual(self.dir(MockMockMock), ["Engine", "Exception", "tests"])

    def testEngine(self):
        self.assertEqual(self.dir(self.mocks), ["alternative", "atomic", "create", "optional", "ordered", "repeated", "tearDown", "unordered"])
        self.assertFalse(isCallable(self.mocks))

    def testMock(self):
        self.assertEqual(self.dir(self.myMock), ["expect", "object"])
        self.assertFalse(isCallable(self.myMock))

    def testExpect(self):
        self.assertEqual(self.dir(self.myMock.expect), [])
        self.assertTrue(isCallable(self.myMock.expect))

    def testExpectation(self):
        self.assertEqual(self.dir(self.myMock.expect.foobar), ["andExecute", "andRaise", "andReturn", "withArguments"])
        self.assertTrue(isCallable(self.myMock.expect.foobar))

    def testCalledExpectation(self):
        self.assertEqual(self.dir(self.myMock.expect.foobar(42)), ["andExecute", "andRaise", "andReturn"])
        self.assertFalse(isCallable(self.myMock.expect.foobar(42)))
        self.assertEqual(self.dir(self.myMock.expect.foobar.withArguments(42)), ["andExecute", "andRaise", "andReturn"])
        self.assertFalse(isCallable(self.myMock.expect.foobar.withArguments(42)))

    def testCalledThenAndedExpectation(self):
        self.assertEqual(self.dir(self.myMock.expect.foobar(42).andReturn(12)), [])
        self.assertFalse(isCallable(self.myMock.expect.foobar(42).andReturn(12)))
        self.assertEqual(self.dir(self.myMock.expect.foobar(42).andRaise(TestException())), [])
        self.assertFalse(isCallable(self.myMock.expect.foobar(42).andRaise(TestException())))
        self.assertEqual(self.dir(self.myMock.expect.foobar(42).andExecute(lambda: 12)), [])
        self.assertFalse(isCallable(self.myMock.expect.foobar(42).andExecute(lambda: 12)))

    def testAndedExpectation(self):
        self.assertEqual(self.dir(self.myMock.expect.foobar.andReturn(12)), [])
        self.assertFalse(isCallable(self.myMock.expect.foobar.andReturn(12)))
        self.assertEqual(self.dir(self.myMock.expect.foobar.andRaise(TestException())), [])
        self.assertFalse(isCallable(self.myMock.expect.foobar.andRaise(TestException())))
        self.assertEqual(self.dir(self.myMock.expect.foobar.andExecute(lambda: 12)), [])
        self.assertFalse(isCallable(self.myMock.expect.foobar.andExecute(lambda: 12)))

    def testObject(self):
        ### @todo Maybe expose expected calls in myMock.object.__dir__
        self.assertEqual(self.dir(self.myMock.object), [])

    def dir(self, o):
        return sorted(a for a in dir(o) if not a.startswith("_"))



# -*- coding: utf-8 -*-

# Copyright 2013 Vincent Jacques
# vincent@vincent-jacques.net

# This file is part of MockMockMock. http://jacquev6.github.com/MockMockMock

# MockMockMock is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

# MockMockMock is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public License along with MockMockMock.  If not, see <http://www.gnu.org/licenses/>.

import unittest

import MockMockMock


class SingleExpectationNotCalled(unittest.TestCase):
    def setUp(self):
        unittest.TestCase.setUp(self)
        self.mocks = MockMockMock.Engine()
        self.myMock = self.mocks.create("myMock")

    def testMethodCallWithBadArgument(self):
        self.myMock.expect.foobar(42)
        with self.assertRaises(MockMockMock.Exception) as cm:
            self.myMock.object.foobar(43)
        self.assertEqual(str(cm.exception), "myMock.foobar called with bad arguments (43,) {}")

    def testMethodCallWithBadName(self):
        self.myMock.expect.foobar()
        with self.assertRaises(MockMockMock.Exception) as cm:
            self.myMock.object.barbaz()
        self.assertEqual(str(cm.exception), "myMock.barbaz called instead of myMock.foobar")

    def testPropertyWithBadName(self):
        self.myMock.expect.foobar.andReturn(42)
        with self.assertRaises(MockMockMock.Exception) as cm:
            self.myMock.object.barbaz
        self.assertEqual(str(cm.exception), "myMock.barbaz called instead of myMock.foobar")

    def testMethodNotCalled(self):
        self.myMock.expect.foobar()
        with self.assertRaises(MockMockMock.Exception) as cm:
            self.mocks.tearDown()
        self.assertEqual(str(cm.exception), "myMock.foobar not called")
        self.myMock.object.foobar()
        self.mocks.tearDown()

    def testPropertyNotCalled(self):
        self.myMock.expect.foobar
        with self.assertRaises(MockMockMock.Exception) as cm:
            self.mocks.tearDown()
        self.assertEqual(str(cm.exception), "myMock.foobar not called")
        self.myMock.object.foobar
        self.mocks.tearDown()

    # def testPropertyNotCalledInOrderedGroup(self):
    #     self.myMock.expect.foobar
    #     self.myMock.expect.barbaz
    #     with self.assertRaises(MockMockMock.Exception) as cm:
    #         self.mocks.tearDown()
    #     self.assertEqual(str(cm.exception), "myMock.foobar not called")
    #     self.myMock.object.foobar
    #     with self.assertRaises(MockMockMock.Exception) as cm:
    #         self.mocks.tearDown()
    #     self.assertEqual(str(cm.exception), "myMock.barbaz not called")
    #     self.myMock.object.barbaz
    #     self.mocks.tearDown()

    # def testPropertyNotCalledInUnorderedGroup(self):
    #     with self.mocks.unordered:
    #         self.myMock.expect.foobar
    #         self.myMock.expect.barbaz
    #     with self.assertRaises(MockMockMock.Exception) as cm:
    #         self.mocks.tearDown()
    #     self.assertEqual(str(cm.exception), "myMock.foobar or myMock.barbaz not called")
    #     self.myMock.object.foobar
    #     with self.assertRaises(MockMockMock.Exception) as cm:
    #         self.mocks.tearDown()
    #     self.assertEqual(str(cm.exception), "myMock.barbaz not called")
    #     self.myMock.object.barbaz
    #     self.mocks.tearDown()



# -*- coding: utf-8 -*-

# Copyright 2013 Vincent Jacques
# vincent@vincent-jacques.net

# This file is part of MockMockMock. http://jacquev6.github.com/MockMockMock

# MockMockMock is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

# MockMockMock is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public License along with MockMockMock.  If not, see <http://www.gnu.org/licenses/>.

from _Details.Mock import Mock
from _Details.ExpectationGrouping import OrderedExpectationGroup, UnorderedExpectationGroup, AtomicExpectationGroup, OptionalExpectationGroup, AlternativeExpectationGroup, RepeatedExpectationGroup
from _Details.ExpectationHandler import ExpectationHandler


class Engine:
    def __init__(self):
        self.__handler = ExpectationHandler(OrderedExpectationGroup())

    def create(self, name):
        return Mock(name, self.__handler)

    def tearDown(self):
        self.__handler.tearDown()

    @property
    def unordered(self):
        return self.__handler.pushGroup(UnorderedExpectationGroup())

    @property
    def ordered(self):
        return self.__handler.pushGroup(OrderedExpectationGroup())

    @property
    def atomic(self):
        return self.__handler.pushGroup(AtomicExpectationGroup())

    @property
    def optional(self):
        return self.__handler.pushGroup(OptionalExpectationGroup())

    @property
    def alternative(self):
        return self.__handler.pushGroup(AlternativeExpectationGroup())

    @property
    def repeated(self):
        return self.__handler.pushGroup(RepeatedExpectationGroup())



# -*- coding: utf-8 -*-

# Copyright 2013 Vincent Jacques
# vincent@vincent-jacques.net

# This file is part of MockMockMock. http://jacquev6.github.com/MockMockMock

# MockMockMock is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

# MockMockMock is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public License along with MockMockMock.  If not, see <http://www.gnu.org/licenses/>.

from MockException import MockException
from Expectation import Expectation
import ArgumentChecking


class AnyAttribute:
    def __init__(self, apply, mockName):
        self.__apply = apply
        self.__mockName = mockName

    def __getattr__(self, name):
        ### @todo Fix this hack
        # Hack to allow m.expect._call_.withArguments,
        # because m.expect.__call__.withArguments fails,
        # because function doesn't have attribute "withArguments"
        if name == "_call_":
            name = "__call__"
        # End of hack
        if name == "__dir__":
            raise AttributeError()
        return self.__apply(self.__mockName + "." + name)

    def __call__(self, *args, **kwds):  # Needed to make isinstance(mock.expect, collections.Callable) return True
        return self.__getattr__("__call__")(*args, **kwds)


class CallChecker:
    def __init__(self, handler, expectations):
        self.__handler = handler
        self.__expectations = expectations

    def __call__(self, *args, **kwds):
        return self.__handler.checkExpectationCall(self.__expectations, args, kwds)


class BasicExpectationProxy:
    def __init__(self, expectation):
        self.__expectation = expectation

    def andReturn(self, value):
        return self.andExecute(lambda: value)

    def andRaise(self, exception):
        def Raise():
            raise exception
        return self.andExecute(Raise)

    def andExecute(self, action):
        self.__expectation.setAction(action)
        return None


class CallableExpectationProxy(BasicExpectationProxy):
    def __init__(self, expectation):
        BasicExpectationProxy.__init__(self, expectation)
        self.__expectation = expectation

    def __call__(self, *args, **kwds):
        return self.withArguments(ArgumentChecking.Equality(args, kwds))

    def withArguments(self, checker):
        self.__expectation.expectCall(checker)
        return BasicExpectationProxy(self.__expectation)


class ExpectationHandler(object):
    def __init__(self, initialGroup):
        self.__currentGroup = initialGroup

    # expect
    def expect(self, mockName):
        return AnyAttribute(self.addExpectation, mockName)

    def addExpectation(self, name):
        # Note that accepting name == "__call__" allows the mock object to be callable with no specific code
        expectation = Expectation(name)
        self.__currentGroup.addExpectation(expectation)
        return CallableExpectationProxy(expectation)

    def pushGroup(self, group):
        self.__currentGroup.addGroup(group)
        self.__currentGroup = group
        return self.StackPoper(self)

    class StackPoper:
        def __init__(self, handler):
            self.__handler = handler

        def __enter__(self):
            pass

        def __exit__(self, *ignored):
            self.__handler.popGroup()

    def popGroup(self):
        self.__currentGroup = self.__currentGroup.parent

    # call
    def object(self, mockName):
        return AnyAttribute(self.checkExpectation, mockName)

    def checkExpectation(self, calledName):
        possibleExpectations = self.__currentGroup.getCurrentPossibleExpectations()
        goodNamedExpectations = [expectation for expectation in possibleExpectations if expectation.checkName(calledName)]

        if len(goodNamedExpectations) == 0:
            raise MockException(calledName + " called instead of " + " or ".join(e.name for e in possibleExpectations))

        allGoodNamedExpectationsExpectNoCall = not any(expectation.expectsCall() for expectation in goodNamedExpectations)
        allGoodNamedExpectationsExpectCall = all(expectation.expectsCall() for expectation in goodNamedExpectations)

        if allGoodNamedExpectationsExpectCall:
            return CallChecker(self, goodNamedExpectations)
        elif allGoodNamedExpectationsExpectNoCall:
            return self.__callExpectation(goodNamedExpectations[0])
        else:
            raise MockException(calledName + " is expected as a property and as a method call in an unordered group")

    def checkExpectationCall(self, expectations, args, kwds):
        for expectation in expectations:
            if expectation.checkCall(args, kwds):
                return self.__callExpectation(expectation)
        raise MockException(expectations[0].name + " called with bad arguments " + str(args) + " " + str(kwds))

    def __callExpectation(self, expectation):
        returnValue, self.__currentGroup = expectation.call()
        return returnValue

    def tearDown(self):
        if self.__currentGroup.requiresMoreCalls():
            raise MockException(" or ".join(self.__currentGroup.getRequiredCallsExamples()) + " not called")



# -*- coding: utf-8 -*-

# Copyright 2013 Vincent Jacques
# vincent@vincent-jacques.net

# This file is part of MockMockMock. http://jacquev6.github.com/MockMockMock

# MockMockMock is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

# MockMockMock is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public License along with MockMockMock.  If not, see <http://www.gnu.org/licenses/>.

"""
MockMockMock
============

Introduction
------------

MockMockMock is a Python `mocking <http://en.wikipedia.org/wiki/Mock_object>`_ library.

It focuses on very explicit definition of the mocks' behaviour.
It allows as-specific-as-needed unit-tests as well as more generic ones.

Basic usage
-----------

This documentation assumes you are fairly confortable with
`unittest <http://docs.python.org/library/unittest.html>`_.

Let's say you are writing a class ``Stats`` that performs basic statistics on integer numbers.
This class needs some source of integers.
Using dependency injection, you pass a ``source`` object to the constructor of ``Stats``.
In real execution, it could read integers from keyboard or from a file.
For your unit-tests, you can mock this source to simulate all kinds of behaviours.

So, here is how you would do that with PyMockMockMock. Create a test case::

    import unittest
    from MockMockMock import Mock

    from Stats import Stats

    class MyTestCase(unittest.TestCase):
        def setUp(self):

In ``setUp``, you create the mock...

::

    .       self.source = Mock("source")

... and inject it in your constructor::

    .       self.stats = Stats(self.source.object)

In ``tearDown``, you check that each mock has been used correctly::

    .   def tearDown(self):
            self.source.tearDown()

Then, in you test methods, there are two phases. First, instruct the mock about how to behave...

::

    .   def testAverage(self):
            self.source.expect.get().andReturn([42, 43, 44])

... then call your code, that will call the mock.

::

    .       self.assertEqual(self.stats.average(), 43)

This way, you:

- are checking that your code calls what it has to call (the final call to Mock.tearDown will detect if some call was ``expect`` ed, but not called)
- can inject to your code wathever behaviour you want to test.

Of course, do not forget to actually run the tests::

    unittest.main()

Mock behaviour
--------------

Expectations
~~~~~~~~~~~~

You can instruct the mock to expect successive calls::

        def testXxxx(self):
            self.source.expect.get().andReturn([1, 2, 3, 4])
            self.source.expect.isFinished().andReturn(True)

            self.assertEqual(self.stats.average(), 2.5)

You can expect successive calls to the same method and have different behaviour each time::

        def testXxxx(self):
            self.source.expect.get().andReturn([1, 2, 3, 4])
            self.source.expect.get().andReturn([5, 6, 7])
            self.source.expect.get().andReturn([])

            self.assertEqual(self.stats.average(), 4)

You can expect arguments in method calls::

        def testXxxx(self):
            self.source.expect.get(5).andReturn([1, 2, 3, 4, 5])

            self.assertEqual(self.stats.average(), 3)

You can expect calls to properties as well::

        def testXxxx(self):
            self.source.expect.name.andReturn("Mock")

            self.assertEqual(self.stats.getSourceName(), "Mock")

Behaviour
~~~~~~~~~

You can simulate a source that raises exceptions::

        def testXxxx(self):
            self.source.expect.get().andRaise(Exception())

            self.assertRaises(Exception, self.stats.average)

Mock grouping
~~~~~~~~~~~~~

By default, all ``expect`` ed calls are registered in an ``ordered`` group.
This means that the mock will raise a ``MockException`` if your code does not call those methods in the order they were ``expect`` ed.
The `Mock.tearDown` method will raise ``MockException`` as if your code does not call all the ``expect`` ed methods.

To tweak this behaviour, there are several other available grouping methods.

You can tell the mock to expect all calls, but in any order::

        def testXxxx(self):
            with self.source.unordered:
                self.source.expect.foobar()
                self.source.expect.barbaz()

            self.stats.frobnicate()

You can tell the mock that some calls are optional::

        def testXxxx(self):
            with self.source.optional:
                self.source.expect.foobar()
                self.source.expect.barbaz()

            self.stats.frobnicate()

Reference
---------
"""

from Engine import Engine
from _Details.MockException import MockException as Exception



# -*- coding: utf-8 -*-

# Copyright 2013 Vincent Jacques
# vincent@vincent-jacques.net

# This file is part of MockMockMock. http://jacquev6.github.com/MockMockMock

# MockMockMock is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

# MockMockMock is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public License along with MockMockMock.  If not, see <http://www.gnu.org/licenses/>.

import unittest

import MockMockMock


groupMakers = {
    "o": lambda f: f.ordered,
    "u": lambda f: f.unordered,
    "a": lambda f: f.atomic,
    "p": lambda f: f.optional,
    "l": lambda f: f.alternative,
    "r": lambda f: f.repeated,
}


def makeExpectations(mocks, mock, groups):
    if len(groups) > 0:
        group = groups[0]
        with groupMakers[group](mocks):
            mock.expect.foobar(group + "A")
            mock.expect.foobar(group + "B")
            makeExpectations(mocks, mock, groups[1:])
            mock.expect.foobar(group + "C")
            mock.expect.foobar(group + "D")


def testAllowedOrder(allowedOrder):
    def test(self):
        for argument in allowedOrder:
            self.call(argument)
        self.mocks.tearDown()
    return test


def testForbidenOrder(forbidenOrder):
    def test(self):
        for argument in forbidenOrder[: -1]:
            self.call(argument)
        with self.assertRaises(MockMockMock.Exception):
            self.call(forbidenOrder[-1])
    return test


def testTearDownError(forbidenOrder):
    def test(self):
        for argument in forbidenOrder:
            self.call(argument)
        with self.assertRaises(MockMockMock.Exception):
            self.mocks.tearDown()
    return test


def makeTestCase(groups, allowedOrders, forbidenOrders, tearDownErrors):
    groups = list(groups)
    allowedOrders = [
        [
            allowedOrder[2 * i:2 * i + 2]
            for i in range(len(allowedOrder) // 2)
        ]
        for allowedOrder in allowedOrders
    ]
    forbidenOrders = [
        [
            forbidenOrder[2 * i:2 * i + 2]
            for i in range(len(forbidenOrder) // 2)
        ]
        for forbidenOrder in forbidenOrders
    ]
    tearDownErrors = [
        [
            tearDownError[2 * i:2 * i + 2]
            for i in range(len(tearDownError) // 2)
        ]
        for tearDownError in tearDownErrors
    ]

    class TestCase(unittest.TestCase):
        def setUp(self):
            unittest.TestCase.setUp(self)
            self.mocks = MockMockMock.Engine()
            self.myMock = self.mocks.create("myMock")
            makeExpectations(self.mocks, self.myMock, groups)

        def call(self, argument):
            assert(len(argument) == 2)
            assert(argument[0] in groupMakers)
            assert(argument[1] in "ABCDX")
            self.myMock.object.foobar(argument)

    for allowedOrder in allowedOrders:
        setattr(TestCase, "test_" + "".join(allowedOrder), testAllowedOrder(allowedOrder))

    for forbidenOrder in forbidenOrders:
        setattr(TestCase, "test_" + "".join(forbidenOrder), testForbidenOrder(forbidenOrder))

    for tearDownError in tearDownErrors:
        setattr(TestCase, "test_" + "".join(tearDownError), testTearDownError(tearDownError))

    TestCase.__name__ = "TestCase_" + "".join(groups)

    return TestCase

UnorderedGroup = makeTestCase(
    "u",
    [
        # Completed in any order
        "uAuBuCuD",
        "uBuAuDuC",
        "uBuCuAuD",
    ],
    [
        # Bad argument
        "uAuX",
    ],
    [
        # Not completed
        "uAuBuC",
        "uAuBuD",
        "uAuB",
        "uA",
        "",
    ]
)

OrderedGroup = makeTestCase(
    "o",
    [
        # Completed in good order
        "oAoBoCoD",
    ],
    [
        # Wrong order
        "oB",
        "oC",
        "oD",
    ],
    [
        # Not completed
        "oAoBoC",
        "oAoB",
        "oA",
        "",
    ]
)

OptionalGroup = makeTestCase(
    "p",
    [
        # Completed in good order
        "pApBpCpD",
        # Not completed
        "pApBpC",
        "pApB",
        "pA",
        "",
    ],
    [
        # Wrong order
        "pB",
    ],
    [
    ]
)

AlternativeGroup = makeTestCase(
    "l",
    [
        # One call
        "lA",
        "lB",
        "lC",
        "lD",
    ],
    [
        # Two calls
        "lAlB",
        "lBlA",
    ],
    [
        # No call
        "",
    ]
)

RepeatedGroup = makeTestCase(
    "r",
    [
        # Zero call
        "",
        # One time
        "rArBrCrD",
        # Several times
        "rArBrCrDrArBrCrD",
        "rArBrCrDrArBrCrDrArBrCrD",
    ],
    [
    ],
    [
        # Partial calls
        "rA",
        "rArB",
        "rArBrCrDrA",
        "rArBrCrDrArB",
        "rArBrCrDrArBrCrDrA",
        "rArBrCrDrArBrCrDrArB",
    ]
)

OrderedInUnorderedGroup = makeTestCase(
    "uo",
    [
        # Original order
        "uAuBoAoBoCoDuCuD",
        # Other possible orders
        #  ordered group at once
        "oAoBoCoDuAuBuCuD",
        "uAuBuCuDoAoBoCoD",
        #  ordered group in pieces
        "oAuAoBuBoCuCoDuD",
        "oAuDoBuCoCuBoDuA",
    ],
    [
        # Ordered group in wrong order
        "uAuBoB",
    ],
    [
    ]
)

AtomicInUnorderedGroup = makeTestCase(
    "ua",
    [
        # Original order
        "uAuBaAaBaCaDuCuD",
        # Other possible orders
        #  atomic group at once
        "aAaBaCaDuAuBuCuD",
        "uAuBuCuDaAaBaCaD",
    ],
    [
        # Atomic group in wrong order
        "uAuBaB",
        # Atomic group in pieces
        "uAaAuB",
    ],
    [
    ]
)

UnorderedInRepeatedGroup = makeTestCase(
    "ru",
    [
        "",
        "rArBuAuBuCuDrCrD",
        "rArBuAuBuCuDrCrDrArBuAuBuCuDrCrD",
    ],
    [
        # Partial
        "rArBuAuBuCrC",
    ],
    [
        # Partial
        "rArBuAuBuCuDrC",
    ]
)



# -*- coding: utf-8 -*-

# Copyright 2013 Vincent Jacques
# vincent@vincent-jacques.net

# This file is part of MockMockMock. http://jacquev6.github.com/MockMockMock

# MockMockMock is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

# MockMockMock is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public License along with MockMockMock.  If not, see <http://www.gnu.org/licenses/>.

import unittest

import MockMockMock


class SequenceBetweenSeveralLinkedMocks(unittest.TestCase):
    def setUp(self):
        unittest.TestCase.setUp(self)
        self.mocks = MockMockMock.Engine()
        self.m1 = self.mocks.create("m1")
        self.m2 = self.mocks.create("m2")

    def testShortCorrectSequence(self):
        self.m1.expect.foobar(42)
        self.m2.expect.foobar(43)
        self.m1.object.foobar(42)
        self.m2.object.foobar(43)
        self.mocks.tearDown()

    def testShortInvertedSequence(self):
        self.m1.expect.foobar(42)
        self.m2.expect.foobar(43)
        with self.assertRaises(MockMockMock.Exception) as cm:
            self.m2.object.foobar(43)
        self.assertEqual(str(cm.exception), "m2.foobar called instead of m1.foobar")



# -*- coding: utf-8 -*-

# Copyright 2013 Vincent Jacques
# vincent@vincent-jacques.net

# This file is part of MockMockMock. http://jacquev6.github.com/MockMockMock

# MockMockMock is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

# MockMockMock is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public License along with MockMockMock.  If not, see <http://www.gnu.org/licenses/>.

import unittest

import MockMockMock


class TestException(Exception):
    pass


class SingleExpectation(unittest.TestCase):
    def setUp(self):
        unittest.TestCase.setUp(self)
        self.mocks = MockMockMock.Engine()
        self.myMock = self.mocks.create("myMock")

    def tearDown(self):
        self.mocks.tearDown()

    def testMethodCall(self):
        self.myMock.expect.foobar()
        self.myMock.object.foobar()

    def testMethodCallWithSimpleArgument(self):
        self.myMock.expect.foobar(42)
        self.myMock.object.foobar(42)

    def testMethodCallWithReturn(self):
        returnValue = object()
        self.myMock.expect.foobar().andReturn(returnValue)
        self.assertIs(self.myMock.object.foobar(), returnValue)

    def testPropertyWithReturn(self):
        self.myMock.expect.foobar.andReturn(42)
        self.assertEqual(self.myMock.object.foobar, 42)

    def testObjectCallWithArgumentsAndReturn(self):
        self.myMock.expect(43, 44).andReturn(42)
        self.assertEqual(self.myMock.object(43, 44), 42)

    def testMethodCallWithRaise(self):
        self.myMock.expect.foobar().andRaise(TestException())
        with self.assertRaises(TestException):
            self.myMock.object.foobar()

    def testPropertyWithRaise(self):
        self.myMock.expect.foobar.andRaise(TestException())
        with self.assertRaises(TestException):
            self.myMock.object.foobar

    def testMethodCallWithSpecificAction(self):
        self.check = False

        def f():
            self.check = True

        self.myMock.expect.foobar().andExecute(f)
        self.myMock.object.foobar()
        self.assertTrue(self.check)

    def testPropertyWithSpecificAction(self):
        self.check = False

        def f():
            self.check = True

        self.myMock.expect.foobar.andExecute(f)
        self.myMock.object.foobar
        self.assertTrue(self.check)



# -*- coding: utf-8 -*-

# Copyright 2013 Vincent Jacques
# vincent@vincent-jacques.net

# This file is part of MockMockMock. http://jacquev6.github.com/MockMockMock

# MockMockMock is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

# MockMockMock is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public License along with MockMockMock.  If not, see <http://www.gnu.org/licenses/>.

import unittest

import MockMockMock
import MockMockMock._Details.ArgumentChecking


class ArgumentCheckers(unittest.TestCase):
    def testCheckerIsUsedByCall(self):
        # We use a myMock...
        checker = MockMockMock.Engine().create("checker")
        checker.expect((12,), {}).andReturn(True)
        checker.expect((13,), {}).andReturn(False)

        # ...to test a mock!
        m = MockMockMock.Engine().create("m")
        m.expect.foobar.withArguments(checker.object).andReturn(42)
        m.expect.foobar.withArguments(checker.object).andReturn(43)
        self.assertEqual(m.object.foobar(12), 42)
        with self.assertRaises(MockMockMock.Exception) as cm:
            m.object.foobar(13)
        self.assertEqual(str(cm.exception), "m.foobar called with bad arguments (13,) {}")

    # def testIdentityChecker(self):
    # def testTypeChecker(self):
    # def testRangeChecker(self):

    def testEqualityChecker(self):
        c = MockMockMock._Details.ArgumentChecking.Equality((1, 2, 3), {1: 1, 2: 2, 3: 3})
        self.assertTrue(c((1, 2, 3), {1: 1, 2: 2, 3: 3}))
        self.assertFalse(c((1, 2, 3), {1: 1, 2: 2, 3: 4}))
        self.assertFalse(c((1, 2, 4), {1: 1, 2: 2, 3: 3}))



# -*- coding: utf-8 -*-

# Copyright 2013 Vincent Jacques
# vincent@vincent-jacques.net

# This file is part of MockMockMock. http://jacquev6.github.com/MockMockMock

# MockMockMock is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

# MockMockMock is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public License along with MockMockMock.  If not, see <http://www.gnu.org/licenses/>.

import unittest

import AllTests


unittest.main(module=AllTests)



# -*- coding: utf-8 -*-

# Copyright 2013 Vincent Jacques
# vincent@vincent-jacques.net

# This file is part of MockMockMock. http://jacquev6.github.com/MockMockMock

# MockMockMock is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

# MockMockMock is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public License along with MockMockMock.  If not, see <http://www.gnu.org/licenses/>.


class AllCompletionPolicy:
    def requiresMoreCalls(self, expectations):
        for expectation in expectations:
            if expectation.requiresMoreCalls():
                return True
        return False

    def getRequiredCallsExamples(self, expectations):
        required = []
        for expectation in expectations:
            if expectation.requiresMoreCalls():
                required += expectation.getRequiredCallsExamples()
        return required

    def acceptsMoreCalls(self, expectations):
        return any(len(expectation.getCurrentPossibleExpectations()) != 0 for expectation in expectations)

    def markExpectationCalled(self, expectations, expectation):
        pass


class AnyCompletionPolicy:
    def requiresMoreCalls(self, expectations):
        return False

    def acceptsMoreCalls(self, expectations):
        return True

    def markExpectationCalled(self, expectations, expectation):
        pass


class ExactlyOneCompletionPolicy:
    def requiresMoreCalls(self, expectations):
        for expectation in expectations:
            if not expectation.requiresMoreCalls():
                return False
        return True

    def getRequiredCallsExamples(self, expectations):
        return []

    def acceptsMoreCalls(self, expectations):
        return self.requiresMoreCalls(expectations)

    def markExpectationCalled(self, expectations, expectation):
        pass


class RepeatedCompletionPolicy:
    def requiresMoreCalls(self, expectations):
        required = 0
        if expectations[0].called:
            for expectation in expectations:
                required += expectation.requiresMoreCalls()
        return required

    def getRequiredCallsExamples(self, expectations):
        return []

    def acceptsMoreCalls(self, expectations):
        return True

    def markExpectationCalled(self, expectations, expectation):
        if expectation is expectations[-1]:
            for e in expectations:
                e.resetCall()



# -*- coding: utf-8 -*-

# Copyright 2013 Vincent Jacques
# vincent@vincent-jacques.net

# This file is part of MockMockMock. http://jacquev6.github.com/MockMockMock

# MockMockMock is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

# MockMockMock is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public License along with MockMockMock.  If not, see <http://www.gnu.org/licenses/>.

import unittest

import MockMockMock


class Ordering(unittest.TestCase):
    def setUp(self):
        unittest.TestCase.setUp(self)
        self.mocks = MockMockMock.Engine()
        self.myMock = self.mocks.create("myMock")

    def testUnorderedGroupOfSameMethod(self):
        with self.mocks.unordered:
            self.myMock.expect.foobar(1).andReturn(11)
            self.myMock.expect.foobar(1).andReturn(13)
            self.myMock.expect.foobar(2).andReturn(12)
            self.myMock.expect.foobar(1).andReturn(14)
        self.assertEqual(self.myMock.object.foobar(2), 12)
        self.assertEqual(self.myMock.object.foobar(1), 11)
        self.assertEqual(self.myMock.object.foobar(1), 13)
        self.assertEqual(self.myMock.object.foobar(1), 14)
        self.mocks.tearDown()

    ### @todo Allow unordered property and method calls on the same name: difficult
    def testUnorderedGroupOfSameMethodAndProperty(self):
        with self.assertRaises(MockMockMock.Exception) as cm:
            with self.mocks.unordered:
                self.myMock.expect.foobar()
                self.myMock.expect.foobar
            self.myMock.object.foobar
        self.assertEqual(str(cm.exception), "myMock.foobar is expected as a property and as a method call in an unordered group")

    def testUnorderedGroupOfSamePropertyAndMethod(self):
        with self.assertRaises(MockMockMock.Exception) as cm:
            with self.mocks.unordered:
                self.myMock.expect.foobar
                self.myMock.expect.foobar()
            self.myMock.object.foobar()
        self.assertEqual(str(cm.exception), "myMock.foobar is expected as a property and as a method call in an unordered group")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright 2013 Vincent Jacques
# vincent@vincent-jacques.net

# This file is part of MockMockMock. http://jacquev6.github.com/MockMockMock

# MockMockMock is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

# MockMockMock is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public License along with MockMockMock.  If not, see <http://www.gnu.org/licenses/>.

import setuptools
import textwrap

version = "0.3.0"


if __name__ == "__main__":
    setuptools.setup(
        name="MockMockMock",
        version=version,
        description="My mock library. Don't use it (yet)",
        author="Vincent Jacques",
        author_email="vincent@vincent-jacques.net",
        url="http://jacquev6.github.com/MockMockMock",
        long_description=textwrap.dedent("""\
        """),
        packages=[
            "MockMockMock",
            "MockMockMock._Details",
            "MockMockMock._Details.ExpectationGrouping",
            "MockMockMock.tests",
        ],
        package_data={
            "MockMockMock": ["COPYING*"],
        },
        classifiers=[
            "Development Status :: 2 - Pre-Alpha",
            "Intended Audience :: Developers",
            "License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)",
            "Operating System :: OS Independent",
            "Programming Language :: Python",
            "Topic :: Software Development",
        ],
        test_suite="MockMockMock.tests.AllTests",
        use_2to3=True
    )
